#pragma once
#ifndef _FTL_RGBD_STEREOVIDEO_HPP_
#define _FTL_RGBD_STEREOVIDEO_HPP_

#include <ftl/rgbd/source.hpp>
#include <string>

namespace ftl {

namespace rgbd {
namespace detail {

class LocalSource;
class Calibrate;
class Disparity;

/**
 * RGBD source from either a stereo video file with left + right images, or
 * direct from two camera devices. A variety of algorithms are included for
 * calculating disparity, before converting to depth.  Calibration of the images
 * is also performed.
 */
class StereoVideoSource : public detail::Source {
	public:
	explicit StereoVideoSource(ftl::rgbd::Source*);
	StereoVideoSource(ftl::rgbd::Source*, const std::string &);
	~StereoVideoSource();

	void swap();
	bool capture(int64_t ts);
	bool retrieve();
	bool compute(int n, int b);
	bool isReady();
	Camera parameters(channel_t chan);

	//const cv::Mat &getRight() const { return right_; }

	private:
	LocalSource *lsrc_;
	Calibrate *calib_;
	Disparity *disp_;
	
	bool ready_;
	
	cv::cuda::Stream stream_;
	cv::cuda::Stream stream2_;

	// possibly should be made available for other users as well
	// (should be called StereoVideoFrame)
	class Frame {
	public:
		bool left_gray_ready;
		bool right_gray_ready;
		bool optflow_ready;

		cv::cuda::GpuMat left;
		cv::cuda::GpuMat right;
		cv::cuda::GpuMat left_gray;
		cv::cuda::GpuMat right_gray;
		cv::cuda::GpuMat disp;
		cv::cuda::GpuMat depth;
		cv::cuda::GpuMat optflow;
		cv::cuda::GpuMat optflow_; // original lower res optical flow

		Frame() { reset(); }

		void reset()
		{
			left_gray_ready = false;
			right_gray_ready = false;
			optflow_ready = false;
		}
	};

	std::vector<Frame> frames_;

	cv::Mat mask_l_;

#ifdef HAVE_OPTFLOW
	cv::Ptr<cv::cuda::NvidiaOpticalFlow_1_0> nvof_;
#endif

	void init(const std::string &);
};

}
}
}

#endif  // _FTL_RGBD_STEREOVIDEO_HPP_
